/* conversion to and from RFC1123 date strings */
#include <u.h>
#include <libc.h>
#include <bio.h>
#include <xml.h>
#include "wdfs.h"

/*
 * Remember, plan9's atoi() understands leading zeros as octal,
 * so we force the beas for conversion by using strtol(s, nil, 10)
 *
 * When setting times we always use gmtime() GMT/UTC/Zulu time
 * which is what WebDav suhould have specified in the first place.
 *
 * How can I be sure what the Timezone is in northwest Angola,
 * and do they spell the timezone the same way plan9 does?
 * Using localtime for remote files - stupid idea.
 */

static char *Months[] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static char *Weekdays[] = {
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

char *
sec2rfc2518(long sec, char *buf, int len)
{
	Tm *tm;

	/* e.g. 2010-04-06T11:39:46Z */
	tm = gmtime(sec);
	snprint(buf, len, "%d-%d-%dT%d:%d:%dZ",
		tm->year + 1900, tm->mon+1, tm->mday,
		tm->hour, tm->min, tm->sec);
	return buf;
}

long
rfc2518(char *str[6])
{
	Tm tm;

	/* e.g. 2010-04-06T11:39:46Z */
	memset(&tm, 0, sizeof(tm));
	tm.year = strtol(str[0], nil, 10) - 1900;
	tm.mon = strtol(str[1], nil, 10) -1;
	tm.mday = strtol(str[2], nil, 10);
	tm.hour = strtol(str[3], nil, 10);
	tm.min = strtol(str[4], nil, 10);
	tm.sec = strtol(str[5], nil, 10);
	return tm2sec(&tm);	// FIXME: Timezone?
}

char *
sec2rfc2068(long sec, char *buf, int len)
{
	Tm *tm;

	/* e.g. Tue, 23 Mar 2010 10:36:20 GMT */
	tm = gmtime(sec);
	snprint(buf, len, "%s, %d %s %d %d:%d:%d %s",
		Weekdays[tm->wday], tm->mday, Months[tm->mon],
		tm->year + 1900, tm->hour, tm->min, tm->sec, tm->zone);
	return buf;
}

static long
rfc2068(char *str[6])
{
	Tm tm;
	int n, mon;
	char *hms[4];

	if((n = gettokens(str[4], hms, nelem(hms), ":")) != 3)
		sysfatal("rfc2068: bad field count in time (%d != 3)\n", n);

	for(mon = 0; mon < nelem(Months); mon++)
		if(strcmp(str[2], Months[mon]) == 0)
			break;
	if(mon >= nelem(Months))
		sysfatal("rfc2068: %s - unknown month\n", str[2]);
	
	memset(&tm, 0, sizeof(tm));
	tm.mon = mon;
	tm.mday = strtol(str[1], nil, 10);
	tm.year = strtol(str[3], nil, 10) - 1900;
	tm.hour = strtol(hms[0], nil, 10);
	tm.min = strtol(hms[1], nil, 10);
	tm.sec = strtol(hms[2], nil, 10);
	return tm2sec(&tm);
}

long
ctime2sec(char *datetime)
{
	long sec;
	char tmp[64], *str[8];

	strncpy(tmp, datetime, sizeof(tmp));
	tmp[sizeof(tmp)-1] = 0;

	sec = 0;
	if(tokenize(tmp, str, nelem(str)) == 6)
		sec = rfc2068(str);
	else
	if(getfields(tmp, str, nelem(str), 0, "-T:") == 6)
		sec = rfc2518(str);
	else
		fprint(2, "ctime2sec: %s - unknown format\n", datetime);
	return sec;
}
