#include <u.h>
#include <libc.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include <libsec.h>
#include <ctype.h>
#include <flate.h>
#include <xml.h>
#include "wdfs.h"

/* This code lifted from webfs */

static int
dhex(char c)
{
	if('0' <= c && c <= '9')
		return c-'0';
	if('a' <= c && c <= 'f')
		return c-'a'+10;
	if('A' <= c && c <= 'F')
		return c-'A'+10;
	return 0;
}

char*
escapeurl(char *s, int (*needesc)(int))
{
	int n;
	char *t, *u;
	Rune r;
	static char *hex = "0123456789abcdef";

	n = 0;
	for(t=s; *t; t++)
		if((*needesc)(*t))
			n++;

	u = emalloc(strlen(s)+2*n+1);
	t = u;
	for(; *s; s++){
		s += chartorune(&r, s);
		if(r >= 0xFF){
			werrstr("URLs cannot contain Runes > 0xFF");
			free(t);
			return nil;
		}
		if((*needesc)(r)){
			*u++ = '%';
			*u++ = hex[(r>>4)&0xF];
			*u++ = hex[r&0xF];
		}else
			*u++ = r;
	}
	*u = '\0';
	return t;
}

char*
unescapeurl(char *s)
{
	char *r, *w;
	Rune rune;

	r = w = s = estrdup(s);
	while(*r){
		if(*r=='%'){
			r++;
			if(!isxdigit(r[0]) || !isxdigit(r[1])){
				werrstr("bad escape sequence '%.3s' in URL", r);
				return nil;
			}
			if(r[0]=='0' && r[2]=='0'){
				werrstr("escaped NUL in URL");
				return nil;
			}
			rune = (dhex(r[0])<<4)|dhex(r[1]);	/* latin1 */
			w += runetochar(w, &rune);
			r += 2;
		}else
			*w++ = *r++;
	}
	*w = '\0';
	return s;
}

