#include <u.h>
#include <libc.h>

static struct {
	int err;
	char *str;
} Errs[] = {
	/* 1xx info */
	{ 100, "continue sending" },
	{ 102, "in progress" },

	/* 2xx Success */
	{ 200, "ok" },
	{ 201, "created" },
	{ 202, "accepted" },
	{ 203, "not authoritative information" },
	{ 204, "no content" },
	{ 205, "reset content" },		// client should re-request data ?
	{ 206, "partial content" },		// reply to GET with Range: spec
	{ 207, "multi status" },

	/* 3xx Redirection */
	{ 300, "multiple choices" },
	{ 301, "moved permanently" },
	{ 302, "found - moved temporarily" },
	{ 303, "see other - redirected by script" },
	{ 304, "not modified" },
	{ 305, "use proxy" },
	{ 306, "switch proxy - depricated" },
	{ 307, "temporary redirect" },

	/* 4xx Client Error */
	{ 400, "bad request" },
	{ 401, "authentication failed" },
	{ 402, "payment required" },
	{ 403, "forbidden, irresepective of auth" },
	{ 404, "does not exist" },
	{ 405, "method not allowed" },
	{ 406, "not acceptable" },
	{ 407, "proxy authentication required" },
	{ 408, "request timeout" },
	{ 409, "conflict" },
	{ 410, "gone - temporarly unavailable" },
	{ 411, "length required" },
	{ 412, "precondition failed" },
	{ 413, "request too big" },
	{ 414, "request uri too long" },
	{ 415, "unsupported media type" },
	{ 416, "requested range not satisfiable" },
	{ 417, "expectation failed" },
	{ 418, "i'm a teapot - April fool" },
	{ 421, "connections from your ip" },
	{ 422, "request not well-formed" },
	{ 423, "locked" },
	{ 424, "failed dependency" },
	{ 425, "unordered collection" },
	{ 426, "upgrade required - e.g. use TLS" },
	{ 449, "retry with" },
	{ 450, "blocked by parental controls" },

	/* 5xx Server Error */
	{ 500, "internal server error" },
	{ 501, "not implemented" },
	{ 502, "bad gateway" },
	{ 503, "service unavailable" },
	{ 504, "gateway timeout" },
	{ 505, "http version not supported" },
	{ 506, "variant also negotiates - circular references" },
	{ 507, "insufficient storage" },
	{ 509, "bandwidth limit exceeded - Apache special" },
	{ 510, "not extended" },
	{ 530, "user access denied" },
};

void
httperrstr(int err)
{
	int i;
	char *class;

	for(i = 0; i < nelem(Errs); i++)
		if(Errs[i].err == err)
			break;

	if(i >= nelem(Errs)){
		if(err >= 100 && err < 200)
			class = "info";
		else
		if(err >= 200 && err < 300)
			class = "success";
		else
		if(err >= 300 && err < 400)
			class = "redirection";
		else
		if(err >= 400 && err < 500)
			class = "client error";
		else
		if(err >= 500 && err < 600)
			class = "server";
		else
			class = "unknown";
		werrstr("class=%s errno=%d (unknown)", class, err);
		return;
	}

	werrstr("%s", Errs[i].str);
}
