
typedef struct Session Session;
typedef struct Aux Aux;
typedef struct Wdfile Wdfile;
typedef struct Cache Cache;
typedef struct Param Param;

struct Param {
	char *name;
	char *value;
};

struct Cache {
	Cache *next;
	char *path;	/* tail of path on server */
	int dirscan;	/* 0 = stat, 1 = readdir */
	Dir *dir;	/* cached dir entries */
	int ndir;	/* number of dir entries in above */
	long atime;	/* last read */
	long expiry;	/* when this entry is junk */
};

struct Session {
	int fd;		/* network IO */
	char *webfs;	/* webfs mount point */
	int conid;	/* webfs connection ID */
	char *base;	/* base url of webdav server */

	Cache *cache;	/* Dir cache */
	RWLock lock;	/* lock on cache */
	long ncache;	/* number of entries in cache */
};

struct Wdfile {
	Session *sess;
	char *path;	/* path to open file */
	int tmpfd;	/* file descriptor of local cache file */
	int modified;	/* file changed, invalidate cache on close */
	vlong length;	/* file length for PUT */
	char *type;	/* mime type for data being written */
	int partialput;	/* partial puts supported */
	int partialget; /* partial gets supported */
	int mode;	/* open mode */
};
	
struct Aux {
	char *path;	/* path to this file */
	Dir *dir;	/* directory entries */
	int ndir;	/* number of directory entries */
	Wdfile *wdf;
};

extern int Propdebug;
extern int Wddebug;
extern int Partput;
extern int Partget;

/* cache.c */
extern int wd_dirread(Session *s, char *path, int dirscan, Dir **dp);
extern void invalidate(Session *s, char *path);
extern void flushcache(Session *s);

/* datetime.c */
extern char *sec2rfc2518(long sec, char *buf, int len);
extern long rfc2518(char *str[6 ]);
extern char *sec2rfc2068(long sec, char *buf, int len);
extern long ctime2sec(char *datetime);

/* http.c */
extern int exch(Session *s, Param *prm, void *tx, int ntx, int txfd);
extern int contentlength(Session *s);

/* httperrstr.c */
extern void httperrstr(int n);

/* main.c */
extern void freedir(Dir *d, int n);
extern void main(int argc, char **argv);

/* wd.c */
extern Session *wd_dial(char *webfs, char *url);
extern int wd_hangup(Session *s);
extern Wdfile *wd_create(Session *s, char *path, int mode);
extern Wdfile *wd_open(Session *s, char *path, int mode);
extern int wd_pread(Wdfile *wdf, void *buf, long len, uvlong off);
extern int wd_pwrite(Wdfile *wdf, void *buf, long len, uvlong off);
extern int wd_close(Wdfile *wdf);
extern int decdir(Xml *xp, Elem *multi, Dir *d);
extern int _wd_dirread(Session *s, char *path, int dirscan, Dir **dp);
extern int wd_mkcol(Session *s, char *path);
extern int wd_delete(Session *s, char *path);
extern int wd_move(Session *s, char *old, char *new);
extern int wd_wstat(Session *s, char *path, char *uid, vlong length, long mtime, long atime, int mode);

